@@ -11,6 +11,29 @@ use Cache::Simple::TimedExpiry;
 
 use strict;
 
+# {{{ Doc
+
+=head1 NAME
+
+DBIx::SearchBuilder::Record::Cachable - Records with caching behavior
+
+=head1 SYNOPSIS
+
+  package MyRecord;
+  use base qw/DBIx::SearchBuilder::Record::Cachable/;
+
+=head1 DESCRIPTION
+
+This module subclasses the main DBIx::SearchBuilder::Record package to add a caching layer. 
+
+The public interface remains the same, except that records which have been loaded in the last few seconds may be reused by subsequent fetch or load methods without retrieving them from the database.
+
+=head1 METHODS
+
+=cut
+
+# }}}
+
 my %_CACHES = ();
 
 # Function: new
@@ -22,17 +45,6 @@ sub new () {
     my ( $class, @args ) = @_;
     my $self = $class->SUPER::new(@args);
 
-    $self->{'_Class'} =
-      ref($self);    # Cache it since we're gonna look at it a _lot_
-
-    if ( $self->can('_CacheConfig') ) {
-        $self->{'_CacheConfig'} = $self->_CacheConfig();
-    }
-    else {
-        $self->{'_CacheConfig'} = __CachableDefaults::_CacheConfig();
-    }
-
-
     return ($self);
 }
 
@@ -40,22 +52,34 @@ sub _SetupCache {
     my $self  = shift;
     my $cache = shift;
     $_CACHES{$cache} = Cache::Simple::TimedExpiry->new();
-    $_CACHES{$cache}->expire_after( $self->{'_CacheConfig'}{'cache_for_sec'} );
+    $_CACHES{$cache}->expire_after( $self->_CacheConfig->{'cache_for_sec'} );
 }
 
+=head2 FlushCache 
+
+This class method flushes the _global_ DBIx::SearchBuilder::Record::Cachable 
+cache.  All caches are immediately expired.
+
+=cut
+
+sub FlushCache {
+    %_CACHES = ();
+}
+
+
 sub _KeyCache {
     my $self = shift;
-    my $cache =     $self->_Handle->DSN . "-KEYS--" . $self->{'_Class'};
-    $self->_SetupCache($cache)  unless exists ($_CACHES{$cache});
-    return ( $_CACHES{ $cache});
+    my $cache = $self->_Handle->DSN . "-KEYS--" . ($self->{'_Class'} ||= ref($self));
+    $self->_SetupCache($cache) unless exists ($_CACHES{$cache});
+    return ($_CACHES{$cache});
 
 }
 
 sub _RecordCache {
     my $self = shift;
-    my $cache =     $self->_Handle->DSN . "--" . $self->{'_Class'};
-    $self->_SetupCache($cache)  unless exists ($_CACHES{$cache});
-    return ( $_CACHES{ $cache});
+    my $cache = $self->_Handle->DSN . "--" . ($self->{'_Class'} ||= ref($self));
+    $self->_SetupCache($cache) unless exists ($_CACHES{$cache});
+    return ($_CACHES{$cache});
 
 }
 
@@ -158,6 +182,7 @@ sub _expire (\$) {
 sub _fetch () {
     my ( $self, $cache_key ) = @_;
     my $data = $self->_RecordCache->fetch($cache_key) or return;
+
     @{$self}{keys %$data} = values %$data; # deserialize
     return 1;
 
@@ -201,7 +226,8 @@ sub _serialize {
 
 sub _gen_alternate_RecordCache_key {
     my ( $self, %attr ) = @_;
-    my $cache_key = $self->Table() . ':';
+    #return( Storable::nfreeze( %attr));
+   my $cache_key;
     while ( my ( $key, $value ) = each %attr ) {
         $key   ||= '__undef';
         $value ||= '__undef';
@@ -225,7 +251,7 @@ sub _gen_alternate_RecordCache_key {
 
 sub _fetch_RecordCache_key {
     my ($self) = @_;
-    my $cache_key = $self->{'_CacheConfig'}{'cache_key'};
+    my $cache_key = $self->_CacheConfig->{'cache_key'};
     return ($cache_key);
 }
 
@@ -281,7 +307,17 @@ sub _lookup_primary_RecordCache_key {
 
 }
 
-package __CachableDefaults;
+=head2 _CacheConfig 
+
+You can override this method to change the duration of the caching from the default of 5 seconds. 
+
+For example, to cache records for up to 30 seconds, add the following method to your class:
+
+  sub _CacheConfig {
+      { 'cache_for_sec' => 30 }
+  }
+
+=cut
 
 sub _CacheConfig {
     {
@@ -289,4 +325,22 @@ sub _CacheConfig {
         'cache_for_sec' => 5,
     };
 }
+
 1;
+
+__END__
+
+# {{{ POD
+
+=head1 AUTHOR
+
+Matt Knopp <mhat@netlag.com>
+
+=head1 SEE ALSO
+
+L<DBIx::SearchBuilder>, L<DBIx::SearchBuilder::Record>
+
+=cut
+
+# }}}
+
